package com.activiti6.mapper;


import com.activiti6.domain.MyFormDataActiviti;
import com.activiti6.domain.MyFormDataActivitiExample;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface MyFormDataActivitiMapper {
    int countByExample(MyFormDataActivitiExample example);

    int deleteByExample(MyFormDataActivitiExample example);

    int deleteByPrimaryKey(Long fId);

    int insert(MyFormDataActiviti record);

    int insertSelective(MyFormDataActiviti record);

    List<MyFormDataActiviti> selectByExample(MyFormDataActivitiExample example);

    MyFormDataActiviti selectByPrimaryKey(Long fId);

    int updateByExampleSelective(@Param("record") MyFormDataActiviti record, @Param("example") MyFormDataActivitiExample example);

    int updateByExample(@Param("record") MyFormDataActiviti record, @Param("example") MyFormDataActivitiExample example);

    int updateByPrimaryKeySelective(MyFormDataActiviti record);

    int updateByPrimaryKey(MyFormDataActiviti record);

    List<MyFormDataActiviti> taskDoneList(@Param("myFormDataActiviti") MyFormDataActiviti myFormDataActiviti);
}
